@@ -1,5 +1,5 @@
 #! /bin/bash -e
-# $Id: new_release.sh 3546 2015-01-01 02:52:34Z bradbell $
+# $Id: new_release.sh 3589 2015-01-11 14:07:42Z bradbell $
 # -----------------------------------------------------------------------------
 # CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
 #
@@ -23,7 +23,7 @@ echo_eval() {
 # -----------------------------------------------------------------------------
 repository="https://projects.coin-or.org/svn/CppAD"
 stable_version="20150000"
-release='1'
+release='2'
 release_version="$stable_version.$release"
 # -----------------------------------------------------------------------------
 # Check release version
@@ -49,7 +49,10 @@ then
 	echo 'Set new version number with following commands:'
 	echo "	bin/version.sh set $release_version"
 	echo '	bin/version.sh copy'
-	echo 'then test and then commit the changes.'
+	echo 'Then test and then commit the changes.'
+	echo 'Then push changes to github.'
+	echo 'Then execute the following command:'
+	echo "	bin/push_git2svn.sh  stable/$stable_version"
 	exit 1
 fi
 # -----------------------------------------------------------------------------
@@ -61,9 +64,6 @@ fi
 git tag -a \
 	-m "corresponds $repository/releases/$release_version" \
 	$release_version
-#
-# copy committed changes to subversion version
-echo_eval git svn dcommit
 # -----------------------------------------------------------------------------
 msg="Creating releases/$release_version"
 rep_stable="$repository/stable/$stable_version"
