@@ -1,8 +1,8 @@
-/* $Id: solve_callback.hpp 2910 2013-10-07 13:27:58Z bradbell $ */
-# ifndef CPPAD_SOLVE_FULL_INCLUDED
-# define CPPAD_SOLVE_FULL_INCLUDED
+/* $Id: solve_callback.hpp 3275 2014-05-20 17:35:11Z bradbell $ */
+# ifndef CPPAD_SOLVE_CALLBACK_INCLUDED
+# define CPPAD_SOLVE_CALLBACK_INCLUDED
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -19,9 +19,7 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 
 namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 namespace ipopt {
-/*
-\defgroup solve_callback solve_callback.hpp
-\{
+/*!
 \file solve_callback.hpp
 \brief Class that connects ipopt::solve to Ipopt
 */
@@ -119,7 +117,7 @@ private:
 	// Hessian information
 	// ----------------------------------------------------------------------
 	/// Sparsity pattern for Hessian of Lagragian
-	/// \f[ L(x) = \sigma \sum_i f_i (x) + \sum_i \lambad_i  g_i (x) \f]
+	/// \f[ L(x) = \sigma \sum_i f_i (x) + \sum_i \lambda_i  g_i (x) \f]
 	/// If sparse is true, this pattern set by constructor and does not change.
 	/// Otherwise this vector has size zero.
 	CppAD::vector< std::set<size_t> > pattern_hes_;
@@ -1110,7 +1108,6 @@ public:
 	}
 };
 
-/*! \} */
 } // end namespace ipopt
 } // END_CPPAD_NAMESPACE
 
