@@ -1,9 +1,9 @@
-/* $Id: ad_fun.hpp 2994 2013-10-23 15:47:20Z bradbell $ */
+/* $Id: ad_fun.hpp 3301 2014-05-24 05:20:21Z bradbell $ */
 # ifndef CPPAD_AD_FUN_INCLUDED
 # define CPPAD_AD_FUN_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -59,8 +59,6 @@ $end
 
 namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 /*!
-\defgroup ad_fun_hpp ad_fun.hpp
-\{
 \file ad_fun.hpp
 File used to define the ADFun<Base> class.
 */
@@ -84,14 +82,17 @@ private:
 	/// debug checking number of comparision operations that changed
 	size_t compare_change_;
 
-	/// number of taylor_ coefficieint per variable (currently stored)
-	size_t taylor_per_var_;
+	/// number of orders stored in taylor_
+	size_t num_order_taylor_;
 
-	/// number of columns currently allocated for taylor_ array
-	size_t taylor_col_dim_;
+	/// maximum number of orders that will fit in taylor_
+	size_t cap_order_taylor_;
 
-	/// number of rows (variables) in the recording (play_)
-	size_t total_num_var_;
+	/// number of directions stored in taylor_
+	size_t num_direction_taylor_;
+
+	/// number of variables in the recording (play_)
+	size_t num_var_tape_;
 
 	/// tape address for the independent variables
 	CppAD::vector<size_t> ind_taddr_;
@@ -102,15 +103,19 @@ private:
 	/// which dependent variables are actually parameters
 	CppAD::vector<bool>   dep_parameter_;
 
-	/// the operation sequence corresponding to this object
-	player<Base> play_;
-
 	/// results of the forward mode calculations
 	pod_vector<Base> taylor_;
 
 	/// which operations can be conditionally skipped
 	/// Set during forward pass of order zero
-	CppAD::vector<bool> cskip_op_;
+	pod_vector<bool> cskip_op_;
+
+	/// Variable on the tape corresponding to each vecad load operation
+	/// (if zero, the operation corresponds to a parameter).
+	pod_vector<addr_t> load_op_;
+
+	/// the operation sequence corresponding to this object
+	player<Base> play_;
 
 	/// Packed results of the forward mode Jacobian sparsity calculations.
 	/// for_jac_sparse_pack_.n_set() != 0  implies other sparsity results
@@ -196,119 +201,44 @@ private:
 	// ------------------------------------------------------------
 	// Forward mode version of SparseJacobian
 	// (see doxygen in sparse_jacobian.hpp)
-	template <class VectorBase, class VectorSet>
+	template <class VectorBase, class VectorSet, class VectorSize>
 	size_t SparseJacobianFor(
-		const VectorBase&     x               ,
-		VectorSet&            p_transpose     ,
-		VectorBase&           jac             ,
-		sparse_jacobian_work& work
+		const VectorBase&           x               ,
+		      VectorSet&            p_transpose     ,
+		const VectorSize&           row             ,
+		const VectorSize&           col             ,
+		      VectorBase&           jac             ,
+		      sparse_jacobian_work& work
 	);
 	// Reverse mode version of SparseJacobian
 	// (see doxygen in sparse_jacobian.hpp)
-	template <class VectorBase, class VectorSet>
+	template <class VectorBase, class VectorSet, class VectorSize>
 	size_t SparseJacobianRev(
-		const VectorBase&     x               ,
-		VectorSet&            p               ,
-		VectorBase&           jac             ,
-		sparse_jacobian_work& work
-	);
-	// ------------------------------------------------------------
-	// vector of bool version of SparseJacobian
-	// (see doxygen in sparse_jacobian.hpp)
-	template <class VectorBase, class VectorSet>
-	size_t SparseJacobianCase(
-		bool                     set_type    ,
-		const VectorBase&        x           ,
-		const VectorSet&         p           ,
-		VectorBase&              jac         ,
-		sparse_jacobian_work&    work
-	);
-	// vector of std::set<size_t> version of SparseJacobian
-	// (see doxygen in sparse_jacobian.hpp)
-	template <class VectorBase, class VectorSet>
-	size_t SparseJacobianCase(
-		const std::set<size_t>&  set_type    ,
-		const VectorBase&        x           ,
-		const VectorSet&         p           ,
-		VectorBase&              jac         ,
-		sparse_jacobian_work&    work
-	);
-	// vector of bool version of SparseJacobian
-	// (see doxygen in sparse_jacobian.hpp)
-	template <class VectorBase, class VectorSet>
-	void SparseJacobianCase(
-		bool                     set_type    ,
-		const VectorBase&        x           ,
-		const VectorSet&         p           ,
-		VectorBase&              jac
-	);
-	// vector of std::set<size_t> version of SparseJacobian
-	// (see doxygen in sparse_jacobian.hpp)
-	template <class VectorBase, class VectorSet>
-	void SparseJacobianCase(
-		const std::set<size_t>&  set_type    ,
-		const VectorBase&        x           ,
-		const VectorSet&         p           ,
-		VectorBase&              jac
+		const VectorBase&           x               ,
+		      VectorSet&            p               ,
+		const VectorSize&           row             ,
+		const VectorSize&           col             ,
+		      VectorBase&           jac             ,
+		      sparse_jacobian_work& work
 	);
 	// ------------------------------------------------------------
 	// combined sparse_set, sparse_list and sparse_pack version of 
 	// SparseHessian (see doxygen in sparse_hessian.hpp)
-	template <class VectorBase, class VectorSet>
+	template <class VectorBase, class VectorSet, class VectorSize>
 	size_t SparseHessianCompute(
-		const VectorBase&        x           ,
-		const VectorBase&        w           ,
-		VectorSet&               sparsity    ,
-		VectorBase&              hes         ,
-		sparse_hessian_work&     work
-	);
-	// vector of bool version of SparseHessian
-	// (see doxygen in sparse_hessian.hpp)
-	template <class VectorBase, class VectorSet>
-	size_t SparseHessianCase(
-		bool                     set_type    ,
-		const VectorBase&        x           ,
-		const VectorBase&        w           ,
-		const VectorSet&         p           ,
-		VectorBase&              hes         ,
-		sparse_hessian_work&     work
-	);
-	// vector of std::set<size_t> version of SparseHessian
-	// (see doxygen in sparse_hessian.hpp)
-	template <class VectorBase, class VectorSet>
-	size_t SparseHessianCase(
-		const std::set<size_t>&  set_type    ,
-		const VectorBase&        x           ,
-		const VectorBase&        w           ,
-		const VectorSet&         p           ,
-		VectorBase&              hes         ,
-		sparse_hessian_work&     work
-	);
-	// vector of bool version of SparseHessian
-	// (see doxygen in sparse_hessian.hpp)
-	template <class VectorBase, class VectorSet>
-	void SparseHessianCase(
-		bool                     set_type    ,
-		const VectorBase&        x           ,
-		const VectorBase&        w           ,
-		const VectorSet&         p           ,
-		VectorBase&              hes
-	);
-	// vector of std::set<size_t> version of SparseHessian
-	// (see doxygen in sparse_hessian.hpp)
-	template <class VectorBase, class VectorSet>
-	void SparseHessianCase(
-		const std::set<size_t>&  set_type    ,
-		const VectorBase&        x           ,
-		const VectorBase&        w           ,
-		const VectorSet&         p           ,
-		VectorBase&              hes
+		const VectorBase&              x           ,
+		const VectorBase&              w           ,
+		      VectorSet&               sparsity    ,
+		const VectorSize&              row         ,
+		const VectorSize&              col         ,
+		      VectorBase&              hes         ,
+		      sparse_hessian_work&     work
 	);
 // ------------------------------------------------------------
 public:
 	/// copy constructor
 	ADFun(const ADFun& g) 
-	: total_num_var_(0)
+	: num_var_tape_(0)
 	{	CppAD::ErrorHandler::Call(
 		true,
 		__LINE__,
@@ -345,10 +275,15 @@ public:
 	template <typename ADvector>
 	void Dependent(const ADvector &x, const ADvector &y);
 
-	/// forward mode sweep
+	/// forward mode user API, one order multiple directions.
 	template <typename VectorBase>
-	VectorBase Forward(
-		size_t p, const VectorBase& x, std::ostream& s = std::cout);
+	VectorBase Forward(size_t q, size_t r, const VectorBase& x);
+
+	/// forward mode user API, multiple directions one order.
+	template <typename VectorBase>
+	VectorBase Forward(size_t q, 
+		const VectorBase& x, std::ostream& s = std::cout
+	);
 
 	/// reverse mode sweep
 	template <typename VectorBase>
@@ -402,11 +337,11 @@ public:
 
 	/// number of operators in the operation sequence
 	size_t size_op(void) const
-	{	return play_.num_rec_op(); }
+	{	return play_.num_op_rec(); }
 
 	/// number of operator arguments in the operation sequence
 	size_t size_op_arg(void) const
-	{	return play_.num_rec_op_arg(); }
+	{	return play_.num_op_arg_rec(); }
 
 	/// amount of memory required for the operation sequence
 	size_t size_op_seq(void) const
@@ -414,26 +349,33 @@ public:
 
 	/// number of parameters in the operation sequence
 	size_t size_par(void) const
-	{	return play_.num_rec_par(); }
+	{	return play_.num_par_rec(); }
 
-	/// number of taylor_ coefficients currently calculated (per variable)
-	size_t size_taylor(void) const
-	{	return taylor_per_var_; } 
+	/// number taylor coefficient orders calculated 
+	size_t size_order(void) const
+	{	return num_order_taylor_; } 
+
+	/// number taylor coefficient directions calculated 
+	size_t size_direction(void) const
+	{	return num_direction_taylor_; } 
 
 	/// number of characters in the operation sequence
 	size_t size_text(void) const
-	{	return play_.num_rec_text(); }
+	{	return play_.num_text_rec(); }
 
 	/// number of variables in opertion sequence
 	size_t size_var(void) const
-	{	return total_num_var_; }
+	{	return num_var_tape_; }
 
 	/// number of VecAD indices in the operation sequence
 	size_t size_VecAD(void) const
-	{	return play_.num_rec_vecad_ind(); }
+	{	return play_.num_vec_ind_rec(); }
 
-	/// set number of coefficients currently allocated (per variable)
-	void capacity_taylor(size_t per_var);   
+	/// set number of orders currently allocated (user API)
+	void capacity_order(size_t c);
+
+	/// set number of orders and directions currently allocated
+	void capacity_order(size_t c, size_t r);   
 
 	/// number of variables in conditional expressions that can be skipped
 	size_t number_skip(void);   
@@ -558,41 +500,52 @@ public:
 	template <typename ADvector>
 	void Dependent(const ADvector &y);
 
-	/// deprecated: number of variables in opertion sequence
+	/// Deprecated: number of variables in opertion sequence
 	size_t Size(void) const
-	{	return total_num_var_; }
+	{	return num_var_tape_; }
 
-	/// deprecated: # taylor_ coefficients currently stored (per variable)
+	/// Deprecated: # taylor_ coefficients currently stored 
+	/// (per variable,direction)
 	size_t Order(void) const
-	{	return taylor_per_var_ - 1; }
+	{	return num_order_taylor_ - 1; }
 
 	/// Deprecated: amount of memory for this object 
 	/// Note that an approximation is used for the std::set<size_t> memory
 	size_t Memory(void) const
-	{	size_t pervar  = taylor_col_dim_ * sizeof(Base)
+	{	size_t pervar  = cap_order_taylor_ * sizeof(Base)
 		+ for_jac_sparse_pack_.memory()
 		+ 3 * sizeof(size_t) * for_jac_sparse_set_.number_elements();
-		size_t total   = total_num_var_ * pervar + play_.Memory();
+		size_t total   = num_var_tape_  * pervar + play_.Memory();
 		return total;
 	}
 
-	/// deprecated: # taylor_ coefficients stored (per variable)
+	/// Deprecated: # taylor_ coefficient orderss stored 
+	/// (per variable,direction)
 	size_t taylor_size(void) const
-	{	return taylor_per_var_; } 
+	{	return num_order_taylor_; } 
 
-	/// deprecated: Does this AD operation sequence use 
-	//VecAD<Base>::reference operands
+	/// Deprecated: Does this AD operation sequence use 
+	/// VecAD<Base>::reference operands
 	bool use_VecAD(void) const
-	{	return play_.num_rec_vecad_ind() > 0; }
+	{	return play_.num_vec_ind_rec() > 0; }
+
+	/// Deprecated: # taylor_ coefficient orders calculated 
+	/// (per variable,direction)
+	size_t size_taylor(void) const
+	{	return num_order_taylor_; } 
+
+	/// Deprecated: set number of orders currently allocated 
+	/// (per variable,direction)
+	void capacity_taylor(size_t per_var);   
 };
 // ---------------------------------------------------------------------------
 
-/*! \} */
 } // END_CPPAD_NAMESPACE
 
 // non-user interfaces
 # include <cppad/local/forward0sweep.hpp>
-# include <cppad/local/forward_sweep.hpp>
+# include <cppad/local/forward1sweep.hpp>
+# include <cppad/local/forward2sweep.hpp>
 # include <cppad/local/reverse_sweep.hpp>
 # include <cppad/local/for_jac_sweep.hpp>
 # include <cppad/local/rev_jac_sweep.hpp>
