@@ -1,9 +1,9 @@
-/* $Id: fun_eval.hpp 2506 2012-10-24 19:36:49Z bradbell $ */
+/* $Id: fun_eval.hpp 3174 2014-03-10 01:26:06Z bradbell $ */
 # ifndef CPPAD_FUN_EVAL_INCLUDED
 # define CPPAD_FUN_EVAL_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-06 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -24,8 +24,8 @@ $index ADFun, evaluate$$
 $section Evaluate ADFun Functions, Derivatives, and Sparsity Patterns$$
 
 $childtable%
-	cppad/local/forward.hpp%
-	cppad/local/reverse.hpp%
+	omh/forward/forward.omh%
+	omh/reverse/reverse.omh%
 	cppad/local/sparse.hpp
 %$$
 
