@@ -1,9 +1,9 @@
-/* $Id: sparse_jacobian.hpp 2910 2013-10-07 13:27:58Z bradbell $ */
+/* $Id: sparse_jacobian.hpp 3301 2014-05-24 05:20:21Z bradbell $ */
 # ifndef CPPAD_SPARSE_JACOBIAN_INCLUDED
 # define CPPAD_SPARSE_JACOBIAN_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-13 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -12,9 +12,18 @@ the terms of the
 A copy of this license is included in the COPYING file of this distribution.
 Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
+
+// maximum number of sparse directions to compute at the same time
+
+// # define CPPAD_SPARSE_JACOBIAN_MAX_MULTIPLE_DIRECTION 1
+# define CPPAD_SPARSE_JACOBIAN_MAX_MULTIPLE_DIRECTION 64
+
 /*
 $begin sparse_jacobian$$
 $spell
+	cppad
+	colpack
+	cmake
 	recomputed
 	valarray
 	std
@@ -113,15 +122,15 @@ $codei%
 %$$
 (see $cref/VectorSize/sparse_jacobian/VectorSize/$$ below).
 They specify which rows and columns of $latex F^{(1)} (x)$$ are
-returned and in what order.
+computes and in what order.
+Not all the non-zero entries in $latex F^{(1)} (x)$$ need be computed,
+but all the entries specified by $icode row$$ and $icode col$$
+must be possibly non-zero in the sparsity pattern.
 We use $latex K$$ to denote the value $icode%jac%.size()%$$
 which must also equal the size of $icode row$$ and $icode col$$.
 Furthermore,
 for $latex k = 0 , \ldots , K-1$$, it must hold that
 $latex row[k] < m$$ and $latex col[k] < n$$.
-In addition, 
-all of the $latex (row[k], col[k])$$ pairs must correspond to a true value
-in the sparsity pattern $icode p$$.
 
 $head jac$$
 The result $icode jac$$ has prototype
@@ -167,6 +176,21 @@ or any of these values have changed,
 you must first call $icode%work%.clear()%$$
 to inform CppAD that this information needs to be recomputed.
 
+$subhead color_method$$
+The coloring algorithm determines which columns (forward mode)
+or rows (reverse mode) can be computed during the same sweep.
+This field has prototype
+$codep%
+	std::string %work%.color_method
+%$$
+and its default value (after a constructor or $code clear()$$) 
+is $code "cppad"$$.
+If $cref colpack_prefix$$ is specified on the
+$cref/cmake command/cmake/CMake Command/$$ line,
+you can set this method to $code "colpack"$$.
+This value only matters on the first call to $code sparse_jacobian$$
+after the $icode work$$ constructor or a call to $code clear$$.
+
 $head n_sweep$$
 The return value $icode n_sweep$$ has prototype
 $codei%
@@ -175,6 +199,8 @@ $codei%
 If $code SparseJacobianForward$$ ($code SparseJacobianReverse$$) is used, 
 $icode n_sweep$$ is the number of first order forward (reverse) sweeps 
 used to compute the requested Jacobian values. 
+(This is also the number of colors determined by the coloring method
+mentioned above).
 This is proportional to the total work that $code SparseJacobian$$ does, 
 not counting the zero order forward sweep, 
 or the work to combine multiple columns (rows) into a single sweep.
@@ -235,11 +261,10 @@ $end
 ==============================================================================
 */
 # include <cppad/local/std_set.hpp>
+# include <cppad/local/color_general.hpp>
 
 namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 /*!
-\defgroup sparse_jacobian_hpp sparse_jacobian.hpp
-\{
 \file sparse_jacobian.hpp
 Sparse Jacobian driver routine and helper functions.
 */
@@ -250,24 +275,22 @@ recomputed.
 */
 class sparse_jacobian_work {
 	public:
-		/// version of user row array with the extra value m at end
-		CppAD::vector<size_t> user_row;
-		/// version of user col array with the extra value n at end
-		CppAD::vector<size_t> user_col;
-		/// indices that sort the user arrays by row 
-		/// with the extra value K at the end
-		CppAD::vector<size_t> sort_row;
-		/// indices that sort the user arrays by column 
-		/// with the extra value K at the end
-		CppAD::vector<size_t> sort_col;
+		/// Coloring method: color_general_cppad or color_general_colpack
+		/// (this field is set by user)
+		std::string color_method;
+		/// indices that sort the user row and col arrays by color 
+		CppAD::vector<size_t> order;
 		/// results of the coloring algorithm
 		CppAD::vector<size_t> color;
-		/// inform CppAD that this information needs to be recomputed
+	
+		/// constructor
+		sparse_jacobian_work(void) : color_method("cppad")
+		{ }
+		/// reset coloring method to its default and
+		/// inform CppAD that color and order need to be recomputed
 		void clear(void)
-		{	user_row.clear();
-			user_col.clear();
-			sort_row.clear();
-			sort_col.clear();
+		{	color_method = "cppad";
+			order.clear();
 			color.clear();
 		}
 };
@@ -276,18 +299,22 @@ class sparse_jacobian_work {
 Private helper function forward mode cases
 
 \tparam Base
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+is the base type for the recording that is stored in this
+<code>ADFun<Base></code> object.
 
 \tparam VectorBase
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+is a simple vector class with elements of type \a Base.
 
 \tparam VectorSet
 is either \c sparse_pack, \c sparse_set or \c sparse_list.
 
-\param x
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\tparam VectorSize
+is a simple vector class with elements of type \c size_t.
 
-\param p_transpose
+\param x [in]
+is a vector specifing the point at which to compute the Jacobian.
+
+\param p_transpose [in]
 If <code>work.color.size() != 0</code>, 
 then \c p_transpose is not used.
 Otherwise, it is a
@@ -295,35 +322,45 @@ sparsity pattern for the transpose of the Jacobian of this ADFun<Base> object.
 Note that we do not change the values in \c p_transpose,
 but is not \c const because we use its iterator facility.
 
+\param row [in]
+is the vector of row indices for the returned Jacobian values.
 
-\param jac
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\param col [in]
+is the vector of columns indices for the returned Jacobian values.
+It must have the same size as \c row.
+
+\param jac [out]
+is the vector of Jacobian values. We use \c K to denote the size of \c jac.
+The return value <code>jac[k]</code> is the partial of the
+<code>row[k]</code> range component of the function with respect
+the the <code>col[k]</code> domain component of its argument.
 
 \param work
-See \c SparseJacobianForward(x, p, row, col, jac, work).
-In addition, \c work.color contains information that is computed by
-\c SparseJacobainFor.
-If the sparsity pattern, \c row vector, and \c col vectors
-are not the same between calls, \c work.clear() must be called
-to reinitialize \c work.color.
+<code>work.color_method</code> is an input. The rest of
+this structure contains information that is computed by \c SparseJacobainFor.
+If the sparsity pattern, \c row vector, or \c col vectors
+are not the same between calls to \c SparseJacobianFor, 
+\c work.clear() must be called to reinitialize \c work.
 
 \return
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+Is the number of first order forward sweeps used to compute the
+requested Jacobian values. The total work, not counting the zero order
+forward sweep, or the time to combine computations, is proportional to this
+return value.
 */
 template<class Base>
-template <class VectorBase, class VectorSet>
+template <class VectorBase, class VectorSet, class VectorSize>
 size_t ADFun<Base>::SparseJacobianFor(
-	const VectorBase&     x           ,
-	VectorSet&            p_transpose ,
-	VectorBase&           jac         ,
-	sparse_jacobian_work& work        )
+	const VectorBase&            x           ,
+	      VectorSet&             p_transpose ,
+	const VectorSize&            row         ,
+	const VectorSize&            col         ,
+	      VectorBase&            jac         ,
+	       sparse_jacobian_work& work        )
 {
-	using   CppAD::vectorBool;
-	size_t i, j, k, ell;
+	size_t j, k, ell;
 
-	CppAD::vector<size_t>& row(work.user_row);
-	CppAD::vector<size_t>& col(work.user_col);
-	CppAD::vector<size_t>& sort_col(work.sort_col);
+	CppAD::vector<size_t>& order(work.order);
 	CppAD::vector<size_t>& color(work.color);
 
 	size_t m = Range();
@@ -341,215 +378,186 @@ size_t ADFun<Base>::SparseJacobianFor(
 
 	// number of components of Jacobian that are required
 	size_t K = size_t(jac.size());
-	CPPAD_ASSERT_UNKNOWN( row.size() == K+1 );
-	CPPAD_ASSERT_UNKNOWN( col.size() == K+1 );
-	CPPAD_ASSERT_UNKNOWN( row[K] == m );
-	CPPAD_ASSERT_UNKNOWN( col[K] == n );
+	CPPAD_ASSERT_UNKNOWN( row.size() == K );
+	CPPAD_ASSERT_UNKNOWN( col.size() == K );
 
 	// Point at which we are evaluating the Jacobian
 	Forward(0, x);
 
+	// check for case where nothing (except Forward above) to do
+	if( K == 0 )
+		return 0;
+
 	if( color.size() == 0 )
 	{
 		CPPAD_ASSERT_UNKNOWN( p_transpose.n_set() ==  n );
 		CPPAD_ASSERT_UNKNOWN( p_transpose.end() ==  m );
 
-		// rows and columns that are in the returned jacobian
-		VectorSet r_used, c_used;
-		r_used.resize(n, m);
-		c_used.resize(m, n);
-		k = 0;
-		while( k < K )
-		{	CPPAD_ASSERT_UNKNOWN( 
-				row[sort_col[k]] < m && col[sort_col[k]] < n 
-			);
-			CPPAD_ASSERT_UNKNOWN( 
-				k == 0 || col[sort_col[k-1]] <= col[sort_col[k]] 
-			);
-			CPPAD_ASSERT_KNOWN(
-				p_transpose.is_element(col[sort_col[k]], row[sort_col[k]]) ,
-				"SparseJacobianForward: "
-				"an (row, col) pair is not in sparsity pattern."
-			);
-			r_used.add_element(col[sort_col[k]], row[sort_col[k]]);
-			c_used.add_element(row[sort_col[k]], col[sort_col[k]]);
-			k++;
-		}
-	
-		// given a row index, which columns are non-zero and not used
-		VectorSet not_used;
-		not_used.resize(m, n);
-		for(j = 0; j < n; j++)
-		{	p_transpose.begin(j);
-			i = p_transpose.next_element();
-			while( i != p_transpose.end() )
-			{	if( ! c_used.is_element(i, j) )
-					not_used.add_element(i, j);
-				i = p_transpose.next_element();
-			}
-		}
-
-		// initial coloring
+		// execute coloring algorithm
 		color.resize(n);
-		for(j = 0; j < n; j++)
-			color[j] = j;
-	
-		// See GreedyPartialD2Coloring Algorithm Section 3.6.2 of
-		// Graph Coloring in Optimization Revisited by
-		// Assefaw Gebremedhin, Fredrik Maane, Alex Pothen
-		vectorBool forbidden(n);
-		for(j = 1; j < n; j++)
+		if(	work.color_method == "cppad" )
+			color_general_cppad(p_transpose, col, row, color);
+		else if( work.color_method == "colpack" )
 		{
-			// initial all colors as ok for this column
-			// (value of forbidden for ell > j does not matter)
-			for(ell = 0; ell <= j; ell++)
-				forbidden[ell] = false;
-	
-			// for each row that is non-zero for this column
-			p_transpose.begin(j);
-			i = p_transpose.next_element();
-			while( i != p_transpose.end() )
-			{	// for each column that this row uses
-				c_used.begin(i);
-				ell = c_used.next_element();
-				while( ell != c_used.end() )
-				{	// if this is not the same column, forbid its color
-					if( ell < j )
-						forbidden[ color[ell] ] = true;
-					ell = c_used.next_element();
-				}
-				i = p_transpose.next_element();
-			}
-	
-			// for each row that this column uses
-			r_used.begin(j);
-			i = r_used.next_element();
-			while( i != r_used.end() )
-			{	// For each column that is non-zero for this row
-				// (the used columns have already been checked above).
-				not_used.begin(i);
-				ell = not_used.next_element();
-				while( ell != not_used.end() )
-				{	// if this is not the same column, forbid its color
-					if( ell < j )
-						forbidden[ color[ell] ] = true;
-					ell = not_used.next_element();
-				}
-				i = r_used.next_element();
-			}
-
-			// pick the color with smallest index
-			ell = 0;
-			while( forbidden[ell] )
-			{	ell++;
-				CPPAD_ASSERT_UNKNOWN( ell <= j );
-			}
-			color[j] = ell;
+# if CPPAD_HAS_COLPACK
+			color_general_colpack(p_transpose, col, row, color);
+# else
+			CPPAD_ASSERT_KNOWN(
+				false,
+				"SparseJacobianForward: work.color_method = colpack "
+				"and colpack_prefix missing from cmake command line."
+			);
+# endif
 		}
+		else CPPAD_ASSERT_KNOWN(
+			false,
+			"SparseJacobianForward: work.color_method is not valid."
+		);
+
+		// put sorting indices in color order
+		VectorSize key(K);
+		order.resize(K);
+		for(k = 0; k < K; k++)
+			key[k] = color[ col[k] ];
+		index_sort(key, order);
 	}
 	size_t n_color = 1;
-	for(ell = 0; ell < n; ell++) 
-		n_color = std::max(n_color, color[ell] + 1);
-
-	// direction vector for calls to forward
-	VectorBase dx(n);
-
-	// location for return values from forward
-	VectorBase dy(m);
+	for(j = 0; j < n; j++) if( color[j] < n )
+		n_color = std::max(n_color, color[j] + 1);
 
 	// initialize the return value
 	for(k = 0; k < K; k++)
 		jac[k] = zero;
 
-	// loop over colors
-	size_t n_sweep = 0;
-	for(ell = 0; ell < n_color; ell++)
-	{	bool any = false;
-		k = 0;
-		for(j = 0; j < n; j++) if( color[j] == ell )
-		{	// find first k such that col[sort_col[k]] has color ell
-			if( ! any )
-			{	while( col[sort_col[k]] < j )
-					k++;
-				any = col[sort_col[k]] == j;
-			}
-		}
-		if( any )
-		{	n_sweep++;
-			// combine all columns with this color
-			for(j = 0; j < n; j++)
-			{	dx[j] = zero;
-				if( color[j] == ell )
-					dx[j] = one;
-			}
-			// call forward mode for all these columns at once
-			dy = Forward(1, dx);
+# if CPPAD_SPARSE_JACOBIAN_MAX_MULTIPLE_DIRECTION == 1
+	// direction vector and return values for calls to forward
+	VectorBase dx(n), dy(m);
 
-			// set the corresponding components of the result
-			for(j = 0; j < n; j++) if( color[j] == ell )
-			{	// find first index in c for this column
-				while( col[sort_col[k]] < j )
-					k++;
-				// extract the row results for this column
-				while( col[sort_col[k]] == j ) 
-				{	jac[ sort_col[k] ] = dy[row[sort_col[k]]];
-					k++;
-				}
-			}
+	// loop over colors
+	k = 0;
+	for(ell = 0; ell < n_color; ell++)
+	{	CPPAD_ASSERT_UNKNOWN( color[ col[ order[k] ] ] == ell );
+
+		// combine all columns with this color
+		for(j = 0; j < n; j++)
+		{	dx[j] = zero;
+			if( color[j] == ell )
+				dx[j] = one;
+		}
+		// call forward mode for all these columns at once
+		dy = Forward(1, dx);
+
+		// set the corresponding components of the result
+		while( k < K && color[ col[order[k]] ] == ell ) 
+		{	jac[ order[k] ] = dy[row[order[k]]];
+			k++;
 		}
 	}
-	return n_sweep;
+# else
+	// abbreviation for this value
+	size_t max_r = CPPAD_SPARSE_JACOBIAN_MAX_MULTIPLE_DIRECTION;
+	CPPAD_ASSERT_UNKNOWN( max_r > 1 );
+
+	// count the number of colors done so far
+	size_t count_color = 0;
+	// count the sparse matrix entries done so far
+	k = 0;
+	while( count_color < n_color )
+	{	// number of colors we will do this time
+		size_t r = std::min(max_r , n_color - count_color);
+		VectorBase dx(n * r), dy(m * r);
+
+		// loop over colors we will do this tme
+		for(ell = 0; ell < r; ell++) 	
+		{	// combine all columns with this color
+			for(j = 0; j < n; j++)
+			{	dx[j * r + ell] = zero;
+				if( color[j] == ell + count_color )
+					dx[j * r + ell] = one;
+			}
+		}
+		size_t q           = 1;
+		dy = Forward(q, r, dx);
+
+		// store results
+		for(ell = 0; ell < r; ell++) 	
+		{	// set the components of the result for this color
+			while( k < K && color[ col[order[k]] ] == ell + count_color ) 
+			{	jac[ order[k] ] = dy[ row[order[k]] * r + ell ];
+				k++;
+			}
+		}
+		count_color += r;
+	}
+# endif
+	return n_color;
 }
 /*!
 Private helper function for reverse mode cases.
 
 \tparam Base
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+is the base type for the recording that is stored in this
+<code>ADFun<Base></code> object.
 
 \tparam VectorBase
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+is a simple vector class with elements of type \a Base.
 
 \tparam VectorSet
 is either \c sparse_pack, \c sparse_set or \c sparse_list.
 
-\param x
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\tparam VectorSize
+is a simple vector class with elements of type \c size_t.
 
-\param p
+\param x [in]
+is a vector specifing the point at which to compute the Jacobian.
+
+\param p [in]
 If <code>work.color.size() != 0</code>, then \c p is not used.
 Otherwise, it is a
 sparsity pattern for the Jacobian of this ADFun<Base> object.
-Note that we do not change the values in \c p_transpose,
+Note that we do not change the values in \c p,
 but is not \c const because we use its iterator facility.
 
-\param jac
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\param row [in]
+is the vector of row indices for the returned Jacobian values.
+
+\param col [in]
+is the vector of columns indices for the returned Jacobian values.
+It must have the same size as \c row.
+
+\param jac [out]
+is the vector of Jacobian values.
+It must have the same size as \c row. 
+The return value <code>jac[k]</code> is the partial of the
+<code>row[k]</code> range component of the function with respect
+the the <code>col[k]</code> domain component of its argument.
 
 \param work
-See \c SparseJacobianReverse(x, p, row, col, jac, work).
-In addition, \c work.color contains information that is computed by
-\c SparseJacobainRev.
-If the sparsity pattern, \c row vector, and \c col vectors
-are not the same between calls, \c work.clear() must be called
-to reinitialize \c work.color.
+<code>work.color_method</code> is an input. The rest of
+This structure contains information that is computed by \c SparseJacobainRev.
+If the sparsity pattern, \c row vector, or \c col vectors
+are not the same between calls to \c SparseJacobianRev, 
+\c work.clear() must be called to reinitialize \c work.
 
 \return
-See \c SparseJacobianReverse(x, p, row, col, jac, work).
+Is the number of first order reverse sweeps used to compute the
+reverse Jacobian values. The total work, not counting the zero order
+forward sweep, or the time to combine computations, is proportional to this
+return value.
 */
 template<class Base>
-template <class VectorBase, class VectorSet>
+template <class VectorBase, class VectorSet, class VectorSize>
 size_t ADFun<Base>::SparseJacobianRev(
-	const VectorBase&     x           ,
-	VectorSet&            p           ,
-	VectorBase&           jac         ,
-	sparse_jacobian_work& work        )
+	const VectorBase&           x           ,
+	      VectorSet&            p           ,
+	const VectorSize&           row         ,
+	const VectorSize&           col         ,
+	      VectorBase&           jac         ,
+	      sparse_jacobian_work& work        )
 {
-	using   CppAD::vectorBool;
-	size_t i, j, k, ell;
+	size_t i, k, ell;
 
-	CppAD::vector<size_t>& row(work.user_row);
-	CppAD::vector<size_t>& col(work.user_col);
-	CppAD::vector<size_t>& sort_row(work.sort_row);
+	CppAD::vector<size_t>& order(work.order);
 	CppAD::vector<size_t>& color(work.color);
 
 	size_t m = Range();
@@ -567,119 +575,52 @@ size_t ADFun<Base>::SparseJacobianRev(
 
 	// number of components of Jacobian that are required
 	size_t K = size_t(jac.size());
-	CPPAD_ASSERT_UNKNOWN( row.size() == K+1 );
-	CPPAD_ASSERT_UNKNOWN( col.size() == K+1 );
-	CPPAD_ASSERT_UNKNOWN( row[K] == m );
-	CPPAD_ASSERT_UNKNOWN( col[K] == n );
+	CPPAD_ASSERT_UNKNOWN( row.size() == K );
+	CPPAD_ASSERT_UNKNOWN( col.size() == K );
 
 	// Point at which we are evaluating the Jacobian
 	Forward(0, x);
 
+	// check for case where nothing (except Forward above) to do
+	if( K == 0 )
+		return 0;
+
 	if( color.size() == 0 )
-	{	CPPAD_ASSERT_UNKNOWN( p.n_set() ==  m );
-		CPPAD_ASSERT_UNKNOWN( p.end() ==  n );
+	{
+		CPPAD_ASSERT_UNKNOWN( p.n_set() == m );
+		CPPAD_ASSERT_UNKNOWN( p.end()   == n );
 
-		// rows and columns that are in the returned jacobian
-		VectorSet r_used, c_used;
-		r_used.resize(n, m);
-		c_used.resize(m, n);
-		k = 0;
-		while( k < K )
-		{	CPPAD_ASSERT_UNKNOWN( 
-				row[sort_row[k]] < m && col[sort_row[k]] < n 
-			);
-			CPPAD_ASSERT_UNKNOWN( 
-				k == 0 || row[sort_row[k-1]] <= row[sort_row[k]] 
-			);
-			CPPAD_ASSERT_KNOWN(
-				p.is_element(row[sort_row[k]], col[sort_row[k]]) ,
-				"SparseJacobianReverse: "
-				"an (row, col) pair is not in sparsity pattern."
-			);
-			r_used.add_element(col[sort_row[k]], row[sort_row[k]]);
-			c_used.add_element(row[sort_row[k]], col[sort_row[k]]);
-			k++;
-		}
-	
-		// given a column index, which rows are non-zero and not used
-		VectorSet not_used;
-		not_used.resize(n, m);
-		for(i = 0; i < m; i++)
-		{	p.begin(i);
-			j = p.next_element();
-			while( j != p.end() )
-			{	if( ! r_used.is_element(j , i) )
-					not_used.add_element(j, i);
-				j = p.next_element();
-			}
-		}
-	
-		// initial coloring
+		// execute the coloring algorithm
 		color.resize(m);
-		for(i = 0; i < m; i++)
-			color[i] = i;
-	
-		// See GreedyPartialD2Coloring Algorithm Section 3.6.2 of
-		// Graph Coloring in Optimization Revisited by
-		// Assefaw Gebremedhin, Fredrik Maane, Alex Pothen
-		vectorBool forbidden(m);
-		for(i = 1; i < m; i++)
+		if(	work.color_method == "cppad" )
+			color_general_cppad(p, row, col, color);
+		else if( work.color_method == "colpack" )
 		{
-			// initial all colors as ok for this row
-			// (value of forbidden for ell > i does not matter)
-			for(ell = 0; ell <= i; ell++)
-				forbidden[ell] = false;
-	
-			// -----------------------------------------------------
-			// Forbid colors for which this row would destroy results 
-			// for each column that is non-zero for this row
-			p.begin(i);
-			j = p.next_element();
-			while( j != p.end() )
-			{	// for each row that this column uses
-				r_used.begin(j);
-				ell = r_used.next_element();
-				while( ell != r_used.end() )
-				{	// if this is not the same row, forbid its color 
-					if( ell < i )
-						forbidden[ color[ell] ] = true;
-					ell = r_used.next_element();
-				}
-				j = p.next_element();
-			}
-
-	
-			// -----------------------------------------------------
-			// Forbid colors that would destroy results for this row.
-			// for each column that this row uses
-			c_used.begin(i);
-			j = c_used.next_element();
-			while( j != c_used.end() )
-			{	// For each row that is non-zero for this column
-				// (the used rows have already been checked above).
-				not_used.begin(j);
-				ell = not_used.next_element();
-				while( ell != not_used.end() )
-				{	// if this is not the same row, forbid its color 
-					if( ell < i )
-						forbidden[ color[ell] ] = true;
-					ell = not_used.next_element();
-				}
-				j = c_used.next_element();
-			}
-
-			// pick the color with smallest index
-			ell = 0;
-			while( forbidden[ell] )
-			{	ell++;
-				CPPAD_ASSERT_UNKNOWN( ell <= i );
-			}
-			color[i] = ell;
+# if CPPAD_HAS_COLPACK
+			color_general_colpack(p, row, col, color);
+# else
+			CPPAD_ASSERT_KNOWN(
+				false,
+				"SparseJacobianReverse: work.color_method = colpack "
+				"and colpack_prefix missing from cmake command line."
+			);
+# endif
 		}
+		else CPPAD_ASSERT_KNOWN(
+			false,
+			"SparseJacobianReverse: work.color_method is not valid."
+		);
+
+		// put sorting indices in color order
+		VectorSize key(K);
+		order.resize(K);
+		for(k = 0; k < K; k++)
+			key[k] = color[ row[k] ];
+		index_sort(key, order);
 	}
 	size_t n_color = 1;
-	for(ell = 0; ell < m; ell++) 
-		n_color = std::max(n_color, color[ell] + 1);
+	for(i = 0; i < m; i++) if( color[i] < m ) 
+		n_color = std::max(n_color, color[i] + 1);
 
 	// weighting vector for calls to reverse
 	VectorBase w(m);
@@ -692,43 +633,26 @@ size_t ADFun<Base>::SparseJacobianRev(
 		jac[k] = zero;
 
 	// loop over colors
-	size_t n_sweep = 0;
+	k = 0;
 	for(ell = 0; ell < n_color; ell++)
-	{	bool any = false;
-		k = 0;
-		for(i = 0; i < m; i++) if( color[i] == ell )
-		{	// find first k such that row[sort_row[k]] has color ell
-			if( ! any )
-			{	while( row[sort_row[k]] < i )
-					k++;
-				any = row[sort_row[k]] == i;
-			}
-		}
-		if( any )
-		{	n_sweep++;
-			// combine all the rows with this color
-			for(i = 0; i < m; i++)
-			{	w[i] = zero;
-				if( color[i] == ell )
-					w[i] = one;
-			}
-			// call reverse mode for all these rows at once
-			dw = Reverse(1, w);
+	{	CPPAD_ASSERT_UNKNOWN( color[ row[ order[k] ] ] == ell );
 
-			// set the corresponding components of the result
-			for(i = 0; i < m; i++) if( color[i] == ell )
-			{	// find first index in r for this row
-				while( row[sort_row[k]] < i )
-					k++;
-				// extract the row results for this row
-				while( row[sort_row[k]] == i ) 
-				{	jac[ sort_row[k] ] = dw[col[sort_row[k]]];
-					k++;
-				}
-			}
+		// combine all the rows with this color
+		for(i = 0; i < m; i++)
+		{	w[i] = zero;
+			if( color[i] == ell )
+				w[i] = one;
+		}
+		// call reverse mode for all these rows at once
+		dw = Reverse(1, w);
+
+		// set the corresponding components of the result
+		while( k < K && color[ row[order[k]] ]  == ell ) 
+		{	jac[ order[k] ] = dw[col[order[k]]];
+			k++;
 		}
 	}
-	return n_sweep;
+	return n_color;
 }
 // ==========================================================================
 // Public Member functions
@@ -743,7 +667,7 @@ The C++ source code corresponding to this operation is
 
 \tparam Base
 is the base type for the recording that is stored in this
-ADFun<Base object.
+<code>ADFun<Base></code> object.
 
 \tparam VectorBase
 is a simple vector class with elements of type \a Base.
@@ -755,40 +679,31 @@ is a simple vector class with elements of type
 \tparam VectorSize
 is a simple vector class with elements of type \c size_t.
 
-\param x
+\param x [in]
 is a vector specifing the point at which to compute the Jacobian.
 
-\param p
+\param p [in]
 is the sparsity pattern for the Jacobian that we are calculating.
 
-\param row
+\param row [in]
 is the vector of row indices for the returned Jacobian values.
 
-\param col
+\param col [in]
 is the vector of columns indices for the returned Jacobian values.
-It must have the same size are r.
+It must have the same size as \c row.
 
-\param jac
+\param jac [out]
 is the vector of Jacobian values.
-It must have the same size are r. 
+It must have the same size as \c row. 
 The return value <code>jac[k]</code> is the partial of the
-<code>row[k]</code> component of the function with respect
-the the <code>col[k]</code> of its argument.
+<code>row[k]</code> range component of the function with respect
+the the <code>col[k]</code> domain component of its argument.
 
-\param work
-\c work contains information that depends on the function object, 
+\param work [in,out]
+this structure contains information that depends on the function object, 
 sparsity pattern, \c row vector, and \c col vector.
-These \c work values are computed by \c SparseJacobianForward.
-If they are not the same between calls, \c work.clear() must be called
-to reinitialize them.
-To be more specific:
-\c user_row is a copy of \c row with the value \c m at the end
-(\c m is the range dimension),
-\c user_col is a copy of \c col with the value \c n at the end
-(\c n is the domain dimension),
-\c sort_col is the index vector that sorts the \c user_col
-vector with the extra value \c K at the end 
-(\c K is the size of \c jac).
+If they are not the same between calls to \c SparseJacobianForward, 
+\c work.clear() must be called to reinitialize them.
 
 \return
 Is the number of first order forward sweeps used to compute the
@@ -808,25 +723,8 @@ size_t ADFun<Base>::SparseJacobianForward(
 {
 	size_t n = Domain();
 	size_t m = Range();
-	size_t k, K = jac.size();
-	if( work.user_row.size() == 0 )
-	{	// create version of (row, col, k) sorted by column value
-		work.user_col.resize(K+1);
-		work.user_row.resize(K+1);
-		work.sort_col.resize(K+1);
-
-		// put sorted indices in user_row and user_col
-		for(k = 0; k < K; k++)
-		{	work.user_row[k] = row[k];
-			work.user_col[k] = col[k];
-		}
-		work.user_row[K] = m;
-		work.user_col[K] = n;
-
-		// put sorting indices in sort_col
-		index_sort(work.user_col, work.sort_col);
-	}
 # ifndef NDEBUG
+	size_t k, K = jac.size();
 	CPPAD_ASSERT_KNOWN(
 		size_t(x.size()) == n ,
 		"SparseJacobianForward: size of x not equal domain dimension for f."
@@ -836,12 +734,6 @@ size_t ADFun<Base>::SparseJacobianForward(
 		"SparseJacobianForward: either r or c does not have "
 		"the same size as jac."
 	); 
-	CPPAD_ASSERT_KNOWN(
-		work.user_row.size() == K+1 &&
-		work.user_col.size() == K+1 &&
-		work.sort_col.size() == K+1 ,
-		"SparseJacobianForward: invalid value in work."
-	);
 	CPPAD_ASSERT_KNOWN(
 		work.color.size() == 0 || work.color.size() == n,
 		"SparseJacobianForward: invalid value in work."
@@ -855,22 +747,10 @@ size_t ADFun<Base>::SparseJacobianForward(
 			col[k] < n,
 			"SparseJacobianForward: invalid value in c."
 		);
-		CPPAD_ASSERT_KNOWN(
-			work.sort_col[k] < K,
-			"SparseJacobianForward: invalid value in work."
-		);
-		CPPAD_ASSERT_KNOWN(
-			work.user_row[k] == row[k],
-			"SparseJacobianForward: invalid value in work."
-		);
-		CPPAD_ASSERT_KNOWN(
-			work.user_col[k] == col[k],
-			"SparseJacobianForward: invalid value in work."
-		);
 	}
 	if( work.color.size() != 0 )
 		for(size_t j = 0; j < n; j++) CPPAD_ASSERT_KNOWN(
-			work.color[j] < n,
+			work.color[j] <= n,
 			"SparseJacobianForward: invalid value in work."
 	);
 # endif
@@ -882,7 +762,7 @@ size_t ADFun<Base>::SparseJacobianForward(
 	{	bool transpose = true;
 		sparsity_user2internal(s_transpose, p, m, n, transpose);
 	}
-	size_t n_sweep = SparseJacobianFor(x, s_transpose, jac, work);
+	size_t n_sweep = SparseJacobianFor(x, s_transpose, row, col, jac, work);
 	return n_sweep;
 }
 /*!
@@ -894,46 +774,44 @@ The C++ source code corresponding to this operation is
 \endverbatim
 
 \tparam Base
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+is the base type for the recording that is stored in this
+<code>ADFun<Base></code> object.
 
 \tparam VectorBase
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+is a simple vector class with elements of type \a Base.
 
 \tparam VectorSet
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+is a simple vector class with elements of type 
+\c bool or \c std::set<size_t>.
 
 \tparam VectorSize
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+is a simple vector class with elements of type \c size_t.
 
-\param x
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\param x [in]
+is a vector specifing the point at which to compute the Jacobian.
 
-\param p
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\param p [in]
+is the sparsity pattern for the Jacobian that we are calculating.
 
-\param row
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\param row [in]
+is the vector of row indices for the returned Jacobian values.
 
-\param col
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\param col [in]
+is the vector of columns indices for the returned Jacobian values.
+It must have the same size as \c row.
 
-\param jac
-See \c SparseJacobianForward(x, p, row, col, jac, work).
+\param jac [out]
+is the vector of Jacobian values.
+It must have the same size as \c row. 
+The return value <code>jac[k]</code> is the partial of the
+<code>row[k]</code> range component of the function with respect
+the the <code>col[k]</code> domain component of its argument.
 
-\param work
-\c work contains information that depends on the function object, 
+\param work [in,out]
+this structure contains information that depends on the function object, 
 sparsity pattern, \c row vector, and \c col vector.
-These \c work values are computed by \c SparseJacobianReverse.
-If they are not the same between calls, \c work.clear() must be called
-to reinitialize them.
-To be more specific:
-\c user_row is a copy of \c row with the value \c m at the end
-(\c m is the range dimension),
-\c user_col is a copy of \c col with the value \c n at the end
-(\c n is the domain dimension),
-\c sort_row is the index vector that sorts the \c user_row
-vector with the extra value \c K at the end 
-(\c K is the size of \c jac).
+If they are not the same between calls to \c SparseJacobianReverse, 
+\c work.clear() must be called to reinitialize them.
 
 \return
 Is the number of first order reverse sweeps used to compute the
@@ -953,25 +831,8 @@ size_t ADFun<Base>::SparseJacobianReverse(
 {
 	size_t m = Range();
 	size_t n = Domain();
-	size_t k, K = jac.size();
-	if( work.user_row.size() == 0 )
-	{	// create version of (row, col, k) sorted by row row value
-		work.user_col.resize(K+1);
-		work.user_row.resize(K+1);
-		work.sort_row.resize(K+1);
-
-		// put sorted indices in user_row and user_col
-		for(k = 0; k < K; k++)
-		{	work.user_row[k] = row[k];
-			work.user_col[k] = col[k];
-		}
-		work.user_row[K] = m;
-		work.user_col[K] = n;
-
-		// put sorting indices in sort_row
-		index_sort(work.user_row, work.sort_row);
-	}
 # ifndef NDEBUG
+	size_t k, K = jac.size();
 	CPPAD_ASSERT_KNOWN(
 		size_t(x.size()) == n ,
 		"SparseJacobianReverse: size of x not equal domain dimension for f."
@@ -981,12 +842,6 @@ size_t ADFun<Base>::SparseJacobianReverse(
 		"SparseJacobianReverse: either r or c does not have "
 		"the same size as jac."
 	); 
-	CPPAD_ASSERT_KNOWN(
-		work.user_row.size() == K+1 &&
-		work.user_col.size() == K+1 &&
-		work.sort_row.size() == K+1 ,
-		"SparseJacobianReverse: invalid value in work."
-	);
 	CPPAD_ASSERT_KNOWN(
 		work.color.size() == 0 || work.color.size() == m,
 		"SparseJacobianReverse: invalid value in work."
@@ -1000,22 +855,10 @@ size_t ADFun<Base>::SparseJacobianReverse(
 			col[k] < n,
 			"SparseJacobianReverse: invalid value in c."
 		);
-		CPPAD_ASSERT_KNOWN(
-			work.sort_row[k] < K,
-			"SparseJacobianReverse: invalid value in work."
-		);
-		CPPAD_ASSERT_KNOWN(
-			work.user_row[k] == row[k],
-			"SparseJacobianReverse: invalid value in work."
-		);
-		CPPAD_ASSERT_KNOWN(
-			work.user_col[k] == col[k],
-			"SparseJacobianReverse: invalid value in work."
-		);
 	}
 	if( work.color.size() != 0 )
 		for(size_t i = 0; i < m; i++) CPPAD_ASSERT_KNOWN(
-			work.color[i] < m,
+			work.color[i] <= m,
 			"SparseJacobianReverse: invalid value in work."
 	);
 # endif
@@ -1027,7 +870,7 @@ size_t ADFun<Base>::SparseJacobianReverse(
 	{	bool transpose = false;
 		sparsity_user2internal(s, p, m, n, transpose);
 	}
-	size_t n_sweep = SparseJacobianRev(x, s, jac, work);
+	size_t n_sweep = SparseJacobianRev(x, s, row, col, jac, work);
 	return n_sweep;
 }
 /*!
@@ -1040,7 +883,7 @@ The C++ source code corresponding to this operation is
 
 \tparam Base
 is the base type for the recording that is stored in this
-ADFun<Base object.
+<code>ADFun<Base></code> object.
 
 \tparam VectorBase
 is a simple vector class with elements of type \a Base.
@@ -1049,10 +892,10 @@ is a simple vector class with elements of type \a Base.
 is a simple vector class with elements of type 
 \c bool or \c std::set<size_t>.
 
-\param x
+\param x [in]
 is a vector specifing the point at which to compute the Jacobian.
 
-\param p
+\param p [in]
 is the sparsity pattern for the Jacobian that we are calculating.
 
 \return
@@ -1086,16 +929,10 @@ VectorBase ADFun<Base>::SparseJacobian(
 			jac[i * n + j] = zero;
 
 	sparse_jacobian_work work;
+	CppAD::vector<size_t> row;
+	CppAD::vector<size_t> col;
 	if( n <= m )
-	{	CppAD::vector<size_t>& row(work.user_row);
-		CppAD::vector<size_t>& col(work.user_col);
-		CppAD::vector<size_t>& sort_col(work.sort_col);
-
-		// forward mode, columns are sorted
-		assert( row.size() == 0 );
-		assert( col.size() == 0 );
-		assert( sort_col.size() == 0 );
-
+	{
 		// need an internal copy of sparsity pattern
 		Pattern_type s_transpose;
 		bool transpose = true;
@@ -1108,34 +945,22 @@ VectorBase ADFun<Base>::SparseJacobian(
 			while( i != s_transpose.end() )
 			{	row.push_back(i);
 				col.push_back(j);
-				sort_col.push_back(k);
 				k++;
 				i = s_transpose.next_element();
 			}
 		} 
 		size_t K = k;
 		VectorBase J(K);
-		row.push_back(m);
-		col.push_back(n);
-		sort_col.push_back(K);
 	
 		// now we have folded this into the following case
-		SparseJacobianFor(x, s_transpose, J, work);
+		SparseJacobianFor(x, s_transpose, row, col, J, work);
 
 		// now set the non-zero return values
 		for(k = 0; k < K; k++)
-			jac[row[sort_col[k]] * n + col[sort_col[k]]] = J[k];
+			jac[ row[k] * n + col[k] ] = J[k];
 	}
 	else
-	{	CppAD::vector<size_t>& row(work.user_row);
-		CppAD::vector<size_t>& col(work.user_col);
-		CppAD::vector<size_t>& sort_row(work.sort_row);
-
-		// reverse mode, rows are sorted
-		assert( row.size() == 0 );
-		assert( col.size() == 0 );
-		assert( sort_row.size() == 0 );
-
+	{
 		// need an internal copy of sparsity pattern
 		Pattern_type s;
 		bool transpose = false;
@@ -1148,23 +973,19 @@ VectorBase ADFun<Base>::SparseJacobian(
 			while( j != s.end() )
 			{	row.push_back(i);
 				col.push_back(j);
-				sort_row.push_back(k);
 				k++;
 				j = s.next_element();
 			}
 		} 
 		size_t K = k;
 		VectorBase J(K);
-		row.push_back(m);
-		col.push_back(n);
-		sort_row.push_back(K);
 
 		// now we have folded this into the following case
-		SparseJacobianRev(x, s, J, work);
+		SparseJacobianRev(x, s, row, col, J, work);
 
 		// now set the non-zero return values
 		for(k = 0; k < K; k++)
-			jac[row[sort_row[k]] * n + col[sort_row[k]]] = J[k];
+			jac[ row[k] * n + col[k] ] = J[k];
 	}
 
 	return jac;
@@ -1180,12 +1001,12 @@ The C++ source code corresponding to this operation is
 
 \tparam Base
 is the base type for the recording that is stored in this
-ADFun<Base object.
+<code>ADFun<Base></code> object.
 
 \tparam VectorBase
 is a simple vector class with elements of the \a Base.
 
-\param x
+\param x [in]
 is a vector specifing the point at which to compute the Jacobian.
 
 \return
@@ -1230,7 +1051,6 @@ VectorBase ADFun<Base>::SparseJacobian( const VectorBase& x )
 	return SparseJacobian(x, p);
 }
 
-
-/*! \} */
 } // END_CPPAD_NAMESPACE
+# undef CPPAD_SPARSE_JACOBIAN_MAX_MULTIPLE_DIRECTION
 # endif
