@@ -1,9 +1,9 @@
-// $Id: std_set.hpp 2910 2013-10-07 13:27:58Z bradbell $
+// $Id: std_set.hpp 3232 2014-04-27 15:38:21Z bradbell $
 # ifndef CPPAD_STD_SET_INCLUDED
 # define CPPAD_STD_SET_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -20,8 +20,6 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 
 namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 /*!
-\defgroup std_set_hpp std_set.hpp
-\{
 \file std_set.hpp
 Two constant standard sets (currently used for concept checking).
 */
@@ -51,6 +49,5 @@ const std::set<Scalar>& two_element_std_set(void)
 	return two;
 }  
 
-/*! \} */
 } // END_CPPAD_NAMESPACE
 # endif
